#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <inttypes.h>

#include <mpi.h>

// acx_mpi_job_count=2

int main(int argc, char **argv) {

  // init mpi
  MPI_Init(&argc, &argv);

  int comm_rank, comm_size;
  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &comm_size);

  if (comm_size != 2) {
    fputs("wrong number of processes\n", stderr);
    MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
  }

  if (comm_rank == 0) {

    {
      unsigned long send_data[2] = {0, ULONG_MAX};
      unsigned long recv_data[2] = {13, 13};
      MPI_Reduce(
        send_data, recv_data, 2, MPI_UNSIGNED_LONG, MPI_MIN, 0, MPI_COMM_WORLD);

      if ((recv_data[0] != 0) || (recv_data[1] != 0)) {
        fprintf(
          stderr, "ERROR in MPI_Reduce (MPI_UNSIGNED_LONG): "
          "recv_data = [%lu; %lu] (expected [0,0])\n",
          recv_data[0], recv_data[1]);
        MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
      }
    }
    {
      uint8_t send_data[2] = {0, UINT8_MAX};
      uint8_t recv_data[2] = {13, 13};
      MPI_Reduce(
        send_data, recv_data, 2, MPI_UINT8_T, MPI_MIN, 0, MPI_COMM_WORLD);

      if ((recv_data[0] != 0) || (recv_data[1] != 0)) {
        fprintf(
          stderr, "ERROR in MPI_Reduce (MPI_UINT8_T): "
          "recv_data = [%"PRIu8"; %"PRIu8"] (expected [0,0])\n",
          recv_data[0], recv_data[1]);
        MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
      }
    }
    {
      uint32_t send_data[2] = {0, UINT32_MAX};
      uint32_t recv_data[2] = {13, 13};
      MPI_Reduce(
        send_data, recv_data, 2, MPI_UINT32_T, MPI_MIN, 0, MPI_COMM_WORLD);

      if ((recv_data[0] != 0) || (recv_data[1] != 0)) {
        fprintf(
          stderr, "ERROR in MPI_Reduce (MPI_UINT32_T): "
          "recv_data = [%"PRIu32"; %"PRIu32"] (expected [0,0])\n",
          recv_data[0], recv_data[1]);
        MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
      }
    }
    {
      uint64_t send_data[2] = {0, UINT64_MAX};
      uint64_t recv_data[2] = {13, 13};
      MPI_Reduce(
        send_data, recv_data, 2, MPI_UINT64_T, MPI_MIN, 0, MPI_COMM_WORLD);

      if ((recv_data[0] != 0) || (recv_data[1] != 0)) {
        fprintf(
          stderr, "ERROR in MPI_Reduce (MPI_UINT64_T): "
          "recv_data = [%"PRIu64"; %"PRIu64"] (expected [0,0])\n",
          recv_data[0], recv_data[1]);
        MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
      }
    }

  } else {

    {
      unsigned long send_data[2] = {ULONG_MAX, 0};
      unsigned long recv_data[2] = {13, 13};
      MPI_Reduce(
        send_data, recv_data, 2, MPI_UNSIGNED_LONG, MPI_MIN, 0, MPI_COMM_WORLD);
    }
    {
      uint8_t send_data[2] = {UINT8_MAX, 0};
      uint8_t recv_data[2] = {13, 13};
      MPI_Reduce(
        send_data, recv_data, 2, MPI_UINT8_T, MPI_MIN, 0, MPI_COMM_WORLD);
    }
    {
      uint32_t send_data[2] = {UINT32_MAX, 0};
      uint32_t recv_data[2] = {13, 13};
      MPI_Reduce(
        send_data, recv_data, 2, MPI_UINT32_T, MPI_MIN, 0, MPI_COMM_WORLD);
    }
    {
      uint64_t send_data[2] = {UINT64_MAX, 0};
      uint64_t recv_data[2] = {13, 13};
      MPI_Reduce(
        send_data, recv_data, 2, MPI_UINT64_T, MPI_MIN, 0, MPI_COMM_WORLD);
    }
  }

  // finalise mpi
  MPI_Barrier(MPI_COMM_WORLD);
  MPI_Finalize();

  return EXIT_SUCCESS;
}
