package de.mpg.mpimet.yacgui.xml.interpolation;

import de.mpg.mpimet.yacgui.xml.Dom;

/**
 * Copyright (c) 2013 Max-Planck-Institute for Meteorology, Hamburg.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Definite interpolation methods for N Nearest Neighbor, Average and Source2Target
 * 
 * @author Maxim Yastremsky, Rene Redler
 */

public class WeightMethods extends Interpolation {
	
	/**
	 * the method of weighted
	 * {@code DISTANCE_WEIGHTED, GAUSS_WEIGHTED, ARITHMETIC_AVERAGE, BARYCENTRIC_COORDINATE}
	 */
	public enum Weighted {
		DISTANCE_WEIGHTED,
		GAUSS_WEIGHTED,
		ARITHMETIC_AVERAGE,
		BARYCENTRIC_COORDINATE;
		
		public static boolean isValidString(String s) {
			return (s.equals(Dom.Value.DISTANCE_WEIGHTED) ||
					s.equals(Dom.Value.GAUSS_WEIGHTED) ||
					s.equals(Dom.Value.BARYCENTRIC_COORDINATE) ||
					s.equals(Dom.Value.ARITHMETIC_AVERAGE));
		}
		public static Weighted objectFromString(String s) {
			if (s.equals(Dom.Value.DISTANCE_WEIGHTED)) {
				return DISTANCE_WEIGHTED;
			} else if (s.equals(Dom.Value.GAUSS_WEIGHTED)) {
				return GAUSS_WEIGHTED;
			} else if (s.equals(Dom.Value.ARITHMETIC_AVERAGE)) {
				return ARITHMETIC_AVERAGE;
			} else if (s.equals(Dom.Value.BARYCENTRIC_COORDINATE)) {
				return BARYCENTRIC_COORDINATE;
			}
			return null; 
		}
		
		@Override
		public String toString() {
			if (this == DISTANCE_WEIGHTED) {
				return Dom.Value.DISTANCE_WEIGHTED;
			} else if (this == GAUSS_WEIGHTED) {
				return Dom.Value.GAUSS_WEIGHTED;
			} else if (this == BARYCENTRIC_COORDINATE) {
				return Dom.Value.BARYCENTRIC_COORDINATE;
			} else if (this == ARITHMETIC_AVERAGE) {
				return Dom.Value.ARITHMETIC_AVERAGE;
			}
			return null;
		}
	}
	
	/**
	 * get attribute name by the order of index
	 * @param index the order of attributes on the panel
	 * @return  the name of attribute
	 * 				if 1 return N, if 2 return WEIGHTED
	 */
	public String getAttributeName(int index) {
		if (index == 0) {
			return Dom.Attribute.GAUSS_SCALE;
		} else if (index == 1) {
			return Dom.Attribute.N;
		} else if (index == 2) {
			return Dom.Attribute.WEIGHTED;
		}
		return null;
	}
	
	private double gausscalefactor;
	private int n;
	private Weighted weighted;

	/**
	 * get attribute value by the order of index
	 * @param index the order of attributes on the panel
	 * @return the value of attribute as String
	 *				if 0 return gauss scale, if 1 return n, if 2 return weighted
	 */
	public String getAttributeValue(int index) {
		if (index == 0) {
			return String.valueOf(gausscalefactor);
		} else if (index == 1) {
			return String.valueOf(n);
		} else if (index == 2) {
			return weighted.toString();
		}
		return null;
	}
}
